﻿namespace TestDetailLevelWrapper
{
    using System;
    using System.Windows;
    using System.Windows.Controls;

    public class DetailLevelWrapper : Control
    {
        #region Constructor

        public DetailLevelWrapper()
        {
            // Set the proper level when the item is loaded
            this.Loaded += delegate { this.SetLevel(); };
        }

        #endregion

        #region Dependency properties

        #region Level dependency property

        public int Level
        {
            get { return (int)this.GetValue(DetailLevelWrapper.LevelProperty); }
            set { this.SetValue(DetailLevelWrapper.LevelProperty, value); }
        }

        public static readonly DependencyProperty LevelProperty =
            DependencyProperty.Register("Level", typeof(int), typeof(DetailLevelWrapper),
                                        new FrameworkPropertyMetadata((int)0, UpdateLevelOnPropertyChange), ValidateLevelValue);

        #endregion

        #region MaxLevel dependency property

        public int MaxLevel
        {
            get { return (int)this.GetValue(DetailLevelWrapper.MaxLevelProperty); }
            set { this.SetValue(DetailLevelWrapper.MaxLevelProperty, value); }
        }

        public static readonly DependencyProperty MaxLevelProperty =
            DependencyProperty.Register("MaxLevel", typeof(int), typeof(DetailLevelWrapper),
                                        new FrameworkPropertyMetadata((int)0, UpdateLevelOnPropertyChange), ValidateLevelValue);

        #endregion

        #region UseMaxLevel dependency property

        public bool UseMaxLevel
        {
            get { return (bool)GetValue(UseMaxLevelProperty); }
            set { SetValue(UseMaxLevelProperty, value); }
        }

        public static readonly DependencyProperty UseMaxLevelProperty =
            DependencyProperty.Register("UseMaxLevel", typeof(bool), typeof(DetailLevelWrapper), 
                                        new FrameworkPropertyMetadata(false, UpdateLevelOnPropertyChange));

        #endregion

        #region ActualLevel dependency property (read-only)

        public int ActualLevel
        {
            get { return (int)this.GetValue(DetailLevelWrapper.ActualLevelProperty); }
        }

        private static readonly DependencyPropertyKey ActualLevelPropertyKey =
            DependencyProperty.RegisterReadOnly("ActualLevel", typeof(int), typeof(DetailLevelWrapper), new UIPropertyMetadata(0));

        public static readonly DependencyProperty ActualLevelProperty = ActualLevelPropertyKey.DependencyProperty;

        #endregion

        #region Level0Template dependency property

        public ControlTemplate Level0Template
        {
            get { return (ControlTemplate)GetValue(Level0TemplateProperty); }
            set { SetValue(Level0TemplateProperty, value); }
        }

        public static readonly DependencyProperty Level0TemplateProperty =
            DependencyProperty.Register("Level0Template", typeof(ControlTemplate), typeof(DetailLevelWrapper), new FrameworkPropertyMetadata(null, UpdateLevelOnPropertyChange));

        #endregion

        #region Level1Template dependency property

        public ControlTemplate Level1Template
        {
            get { return (ControlTemplate)GetValue(Level1TemplateProperty); }
            set { SetValue(Level1TemplateProperty, value); }
        }

        public static readonly DependencyProperty Level1TemplateProperty =
            DependencyProperty.Register("Level1Template", typeof(ControlTemplate), typeof(DetailLevelWrapper), new FrameworkPropertyMetadata(null, UpdateLevelOnPropertyChange));

        #endregion

        #region Level2Template dependency property

        public ControlTemplate Level2Template
        {
            get { return (ControlTemplate)GetValue(Level2TemplateProperty); }
            set { SetValue(Level2TemplateProperty, value); }
        }

        public static readonly DependencyProperty Level2TemplateProperty =
            DependencyProperty.Register("Level2Template", typeof(ControlTemplate), typeof(DetailLevelWrapper), new FrameworkPropertyMetadata(null, UpdateLevelOnPropertyChange));

        #endregion

        #region Level3Template dependency property

        public ControlTemplate Level3Template
        {
            get { return (ControlTemplate)GetValue(Level3TemplateProperty); }
            set { SetValue(Level3TemplateProperty, value); }
        }

        public static readonly DependencyProperty Level3TemplateProperty =
            DependencyProperty.Register("Level3Template", typeof(ControlTemplate), typeof(DetailLevelWrapper), new FrameworkPropertyMetadata(null, UpdateLevelOnPropertyChange));

        #endregion

        #region Level4Template dependency property

        public ControlTemplate Level4Template
        {
            get { return (ControlTemplate)GetValue(Level4TemplateProperty); }
            set { SetValue(Level4TemplateProperty, value); }
        }

        public static readonly DependencyProperty Level4TemplateProperty =
            DependencyProperty.Register("Level4Template", typeof(ControlTemplate), typeof(DetailLevelWrapper), new FrameworkPropertyMetadata(null, UpdateLevelOnPropertyChange));

        #endregion

        #endregion

        #region Internal helper methods

        private static void UpdateLevelOnPropertyChange(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            DetailLevelWrapper _this = (DetailLevelWrapper)o;
            _this.SetLevel();
        }

        private static bool ValidateLevelValue(object value)
        {
            // The value should be an integer
            if (!(value is int))
                return false;

            // Determine the level
            int level = (int)value;
            return (level >= 0) && (level < 5);
        }

        private void SetLevel()
        {
            if (this.IsLoaded)
            {
                // Determine level
                int requestedLevel = this.UseMaxLevel ? this.MaxLevel : this.Level;

                // Check if levels differ (or if template has never been set before)
                if ((this.ActualLevel != requestedLevel) || ((requestedLevel == 0) && (this.Template == null)))
                {
                    // Return the proper template
                    switch (requestedLevel)
                    {
                        case 0: this.Template = this.Level0Template; break;
                        case 1: this.Template = this.Level1Template; break;
                        case 2: this.Template = this.Level2Template; break;
                        case 3: this.Template = this.Level3Template; break;
                        case 4: this.Template = this.Level4Template; break;
                        default: this.Template = null; break;
                    }

                    // Set new value
                    this.SetValue(DetailLevelWrapper.ActualLevelPropertyKey, requestedLevel);
                }
            }
        }

        #endregion
    }
}
